﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace picViewer_NS
{
    /// <summary>
    /// Extend Panel behavior to correctly draw transparent children.
    /// Children draw in inverse z-depth order into the active background.
    /// Each use the background from the previous draw, adding to the background.
    /// 
    /// The code works well for solid color backgrounds.
    /// The results are not perfect when you use a background image (plus it is much slower).
    /// 
    /// Author: Dennis Lang 2009
    /// 
    /// </summary>
    public partial class LayerPanel : Panel
    {
        public LayerPanel()
        {
            InitializeComponent();
            OrgBgImage = this.BackgroundImage;
            base.BackgroundImage = null;
        }

        public override Image BackgroundImage
        {
            get
            {
                return base.BackgroundImage;
            }
            set
            {
                this.OrgBgImage = (value != null) ? new Bitmap(value) : null;
                base.BackgroundImage = value;            
            }
        }

        public bool TransparentChildren
        {
            get { return this.transparentChildren; }
            set { this.transparentChildren = value; }
        }

        bool transparentChildren = true;
        Image OrgBgImage;
        Bitmap bgBitmap;
        Graphics g;

        bool clearBg = true;
        protected override void OnPaintBackground(PaintEventArgs e)
        {
            if (clearBg)
            {
                if (transparentChildren)
                {
                    if (bgBitmap == null)
                    {
                        bgBitmap = new Bitmap(this.Width, this.Height, System.Drawing.Imaging.PixelFormat.Format32bppArgb);
                        g = Graphics.FromImage(bgBitmap);
                    }

                    if (this.OrgBgImage != null)
                    {
                        // Stretch optional background image.
                        GraphicsUnit gUnits = new GraphicsUnit();
                        RectangleF dstRect = this.Bounds;
                        RectangleF srcRect = this.OrgBgImage.GetBounds(ref gUnits);
                        g.DrawImage(this.OrgBgImage, dstRect, srcRect, gUnits);
                    }
                    else
                    {
                        g.Clear(this.BackColor);
                        // g.Clear(Color.Transparent);
                    }
                }
                base.OnPaintBackground(e);
            }
        }

        protected override void OnPaint(PaintEventArgs pe)
        {
            if (transparentChildren)
            {
                transparentChildren = false;    // prevent re-entry
                
                if (bgBitmap == null)
                {
                    bgBitmap = new Bitmap(this.Width, this.Height, System.Drawing.Imaging.PixelFormat.Format32bppArgb);
                    g = Graphics.FromImage(bgBitmap);
                    g.Clear(this.BackColor);
                    // g.Clear(Color.Transparent);
                }

                if (this.OrgBgImage != null)
                {
                    // If background image has any transparent color, call Clear to set background color.
                    // Stretch optional background image.
                    GraphicsUnit gUnits = new GraphicsUnit();
                    RectangleF dstRect = this.Bounds;
                    RectangleF srcRect = this.OrgBgImage.GetBounds(ref gUnits);
                    g.DrawImage(this.OrgBgImage, dstRect, srcRect, gUnits);
                }
                else
                {
                    g.Clear(Color.Transparent);
                }

                clearBg = false;    // prevent children from clearing background
                base.BackgroundImage = bgBitmap;

                // Draw children in reverse order
                for (int cIdx = this.Controls.Count - 1; cIdx >= 0 ; cIdx--)
                {
                    g.Flush();
                    Control c = this.Controls[cIdx];
                    Rectangle controlRect = new Rectangle(c.Location, c.Size);
                    if (this.Bounds.Contains(controlRect))
                    {
                        if (c.Enabled && c.Visible)
                        {
                            c.DrawToBitmap(bgBitmap, controlRect);
                        }
                    }
                }

                clearBg = true;
                transparentChildren = true;
                pe.Graphics.DrawImage(bgBitmap, Point.Empty);
            }
            else
            {
                base.OnPaint(pe);
            }
        }
    }
}
